<?php

/**
 * @file
 * This file implements the DrupalGap service resource call back functions.
 */

/**
 * Determines whether the current user can access a drupalgap resource.
 *
 * @param string $op
 *   String indicating which operation to check access for.
 * @param array $args
 *   Array arguments passed through from the original request.
 *
 * @return boolean
 *   Boolean indicating whether or not the user has access to the resource.
 *
 * @see node_access()
 */
function _drupalgap_resource_access($op = 'view', $args = array()) {
  return user_access($op);
}

/**
 * Returns a collection of content types from the {node_type} table.
 *
 * It also bundles other available information about each content type
 * with the return results.
 *
 * @param array $options
 *   An array of options.
 *
 * @return object
 *   MySQL object results from the {node_type} table
 */
function _drupalgap_resource_content_types_list($options = array()) {
  // Grab content types.
  $sql = " SELECT * FROM {node_type} ORDER BY name ASC ";
  $result = db_query($sql);
  if ($result) {
    $content_types = array();
    while ($data = db_fetch_object($result)) {
      $content_types[] = $data;
    }
    foreach ($content_types as $i => $content_type) {
      // Grab comment settings for content type.
      $names = array(
        'comment_anonymous_' . $content_type->type,
        'comment_' . $content_type->type,
        'comment_default_mode_' . $content_type->type,
        'comment_default_per_page_' . $content_type->type,
        'comment_form_location_' . $content_type->type,
        'comment_preview_' . $content_type->type,
        'comment_subject_field_' . $content_type->type,
      );
      $sql = " SELECT * FROM {variable} ";
      $sql .= " WHERE name IN (" . db_placeholders($names, 'text') . ")";

      // Retrieve the comment settings variables.
      $variable_results = db_query($sql, $names);

      if ($variable_results) {
        // Extract comment settings variables and attach to content type result.
        while ($variable = db_fetch_object($variable_results)) {
          // Strip the '_type' from the end.
          $end = drupal_strlen($variable->name);
          $end -= (drupal_strlen($content_type->type) + 1);
          $variable_name = drupal_substr($variable->name, 0, $end);
          $content_types[$i]->$variable_name = unserialize($variable->value);
        }
      }
    }
    return $content_types;
  }
}

/**
 * Returns a collection of permissions from content types for the current user.
 *
 * @return array
 *   Array of content types with permissions for each.
 */
function _drupalgap_resource_content_types_user_permissions() {
  $content_types = _drupalgap_resource_content_types_list();
  $content_type_permissions = array();
  foreach ($content_types as $content_type) {
    $content_type_permissions[$content_type->type] = array(
      'create' => user_access("create $content_type->type content"),
      'delete any' => user_access("delete any $content_type->type content"),
      'delete own' => user_access("delete own $content_type->type content"),
      'edit any' => user_access("edit any $content_type->type content"),
      'edit own' => user_access("edit own $content_type->type content"),
    );
  }
  return $content_type_permissions;
}

/**
 * Checks to see if the user has access to a permission.
 *
 * @return boolean
 *   Bool indicating whether or not the user has access to the permission.
 *
 * @see user_access()
 */
function _drupalgap_resource_user_access($data) {
  if (!isset($data['permission'])) {
    return services_error(t('Missing argument permission.'), 406);
  }
  return user_access($data['permission']);
}

/**
 * Returns a user's roles and permissions.
 *
 * @return array
 *   Array of user roles and their corresponding permissions.
 */
function _drupalgap_resource_user_roles_and_permissions($data) {
  if (!isset($data['uid'])) {
    services_error(t('Missing argument uid.'), 406);
  }
  $results = array();
  // Load user.
  $temp_user = user_load($data['uid']);
  if ($temp_user) {
    // Extract user role ids.
    $rids = array_keys($temp_user->roles);
    if (!empty($rids)) {
      // Retrieve the permissions for this user role.
      $sql = " SELECT rid, perm FROM {permission} ";
      $sql .= " WHERE rid IN (" . db_placeholders($rids, 'int') . ") ";
      $sql .= " GROUP BY rid ";
      $perms = db_query($sql, $rids);
      if ($perms) {
        while ($data = db_fetch_object($perms)) {
          // Structure the roles and permissions result to match that of D7,
          // so the mobile app only needs to understand one set of data.
          $permissions = explode(", ", $data->perm);
          foreach ($permissions as $permission) {
            $result = new stdClass();
            $result->uid = $temp_user->uid;
            $result->rid = $data->rid;
            $result->name = $temp_user->roles[$data->rid];
            $result->permission = $permission;
            $results[] = $result;
          }
        }
      }
    }
  }
  return $results;
}

/**
 * Returns a collection of variables from the current Drupal site.
 *
 * @return array
 *   Array of variables from the variable table.
 */
function _drupalgap_resource_system_site_settings() {
  // Grab column names from the variable table.
  $names = array(
    'admin_theme',
    'clean_url',
    'date_default_timezone',
    'site_name',
    'theme_default',
    'user_register',
  );
  $sql = " SELECT * FROM {variable} ";
  $sql .= " WHERE name IN (" . db_placeholders($names, 'text') . ")";

  $settings = new stdClass();
  $result = db_query($sql, $names);
  if ($result) {
    $settings->variable = new stdClass();
    while ($variable = db_fetch_object($result)) {
      $name = $variable->name;
      $value = unserialize($variable->value);
      $settings->variable->$name = $value;
    }
  }

  // Add Drupal core verion into settings.
  $settings->variable->drupal_core = "6";

  return $settings;
}
